"""Extract readable data from a HDF5-file and copy it to a new HDF5-file.

:usage:

    G5repair [options] <source> <destination>

:arguments:

    <source>
        Source HDF5-file, possibly containing corrupted data.

    <destination>
        Destination HDF5-file.

:options:

    -f, --force
        Force continuation, overwrite existing files.

    -h, --help
        Show help.

    --version
        Show version.

(c - MIT) T.W.J. de Geus | tom@geus.me | www.geus.me | github.com/tdegeus/GooseHDF5
"""
import argparse
import os
import sys
import warnings

import click
import h5py

from .. import copy
from .. import getdatapaths
from .. import verify
from .. import version

warnings.filterwarnings("ignore")


def check_isfile(fname):
    if not os.path.isfile(fname):
        raise OSError(f'"{fname}" does not exist')


def main():

    try:

        class Parser(argparse.ArgumentParser):
            def print_help(self):
                print(__doc__)

        parser = Parser()
        parser.add_argument("-f", "--force", required=False, action="store_true")
        parser.add_argument("-v", "--version", action="version", version=version)
        parser.add_argument("source")
        parser.add_argument("destination")
        args = parser.parse_args()

        check_isfile(args.source)

        if os.path.isfile(args.destination) and not args.force:
            if not click.confirm(f'File "{args.destination}" exists, continue [y/n]? '):
                sys.exit(1)

        with h5py.File(args.source, "r") as source:

            paths = verify(source, getdatapaths(source))

            with h5py.File(args.destination, "w") as dest:
                copy(source, dest, paths)

    except Exception as e:

        print(e)
        return 1


if __name__ == "__main__":

    main()



import os
import subprocess

import h5py
import numpy as np


def run(cmd):
    out = list(filter(None, subprocess.check_output(cmd, shell=True).decode("utf-8").split("\n")))
    return [i.rstrip().replace("\r", "") for i in out]


a = np.random.random(5)

with h5py.File("a.hdf5", "w") as source:
    source["/a"] = a
    source["/a"].attrs["desc"] = "Example"

expected_output = ["desc : Example"] + [str(a)]

output = run('G5print -a a.hdf5 "/a"')

os.remove("a.hdf5")

if output != expected_output:
    print("output = ")
    print(output)
    print("expected output = ")
    print(expected_output)
    raise OSError("Test failed")



import os
import shutil
import subprocess
import unittest

import h5py
import numpy as np


dirname = os.path.join(os.path.abspath(os.path.dirname(__file__)), "mytest")


def run(cmd):
    ret = subprocess.run(cmd, capture_output=True, text=True)
    return list(filter(None, [i.rstrip().replace("\r", "") for i in ret.stdout.split("\n")]))


class MyTests(unittest.TestCase):
    """ """

    @classmethod
    def setUpClass(self):

        if not os.path.isdir(dirname):
            os.makedirs(dirname)

    @classmethod
    def tearDownClass(self):

        shutil.rmtree(dirname)

    def test_G5list(self):
        """
        Simple tests on G5list
        """

        filepath = os.path.join(dirname, "a.h5")

        with h5py.File(filepath, "w") as file:
            file["/a"] = np.random.random(25)
            file["/b/a"] = np.random.random(25)
            file["/b/b/a"] = np.random.random(25)

        output = sorted(run(["G5list", filepath]))
        expected = sorted(["/a", "/b/a", "/b/b/a"])
        self.assertEqual(output, expected)

    def test_G5list_depth(self):
        """
        Simple tests on G5list
        """

        filepath = os.path.join(dirname, "a.h5")

        with h5py.File(filepath, "w") as file:
            file["/a"] = np.random.random(25)
            file["/b/a"] = np.random.random(25)
            file["/b/b/a"] = np.random.random(25)
            file["/b/c/d/a"] = np.random.random(25)

            g = file.create_group("/c/a")
            g.attrs["foo"] = True

            g = file.create_group("/d/e/a")
            g.attrs["foo"] = True

        output = sorted(run(["G5list", "-d", "1", filepath]))
        expected = sorted(["/a", "/b/...", "/c/...", "/d/..."])
        self.assertEqual(output, expected)

        output = sorted(run(["G5list", "-d", "2", filepath]))
        expected = sorted(["/a", "/b/a", "/b/b/...", "/b/c/...", "/c/a", "/d/e/..."])
        self.assertEqual(output, expected)

        output = sorted(run(["G5list", "-d", "3", filepath]))
        expected = sorted(["/a", "/b/a", "/b/b/a", "/b/c/d/...", "/c/a", "/d/e/a"])
        self.assertEqual(output, expected)

        for i in range(4, 7):
            output = sorted(run(["G5list", "-d", str(i), filepath]))
            expected = sorted(["/a", "/b/a", "/b/b/a", "/b/c/d/a", "/c/a", "/d/e/a"])
            self.assertEqual(output, expected)


if __name__ == "__main__":

    unittest.main()



import os
import subprocess

import h5py
import numpy as np
import prettytable


def run(cmd):
    out = list(filter(None, subprocess.check_output(cmd, shell=True).decode("utf-8").split("\n")))
    return [i.rstrip().replace("\r", "") for i in out]


with h5py.File("a.hdf5", "w") as source:
    with h5py.File("b.hdf5", "w") as other:

        # NumPy array

        a = np.random.random(25)

        source["/a/equal"] = a
        source["/a/not_equal"] = a

        other["/a/equal"] = a
        other["/a/not_equal"] = np.random.random(25)

        # single number

        b = float(np.random.random(1))

        source["/b/equal"] = b
        source["/b/not_equal"] = b

        other["/b/equal"] = b
        other["/b/not_equal"] = float(np.random.random(1))

        # string

        c = "foobar"

        source["/c/equal"] = c
        source["/c/not_equal"] = c

        other["/c/equal"] = c
        other["/c/not_equal"] = "foobar2"

        # alias

        d = np.random.random(25)

        source["/d/equal"] = d
        other["/e/equal"] = d

        # attribute

        f = np.random.random(25)

        source["/f/equal"] = f
        source["/f/equal"].attrs["key"] = f
        source["/f/not_equal"] = f
        source["/f/not_equal"].attrs["key"] = f

        other["/f/equal"] = f
        other["/f/equal"].attrs["key"] = f
        other["/f/not_equal"] = f
        other["/f/not_equal"].attrs["key"] = np.random.random(25)

        # meta (not present)

        meta = source.create_group("/meta")
        meta.attrs["version"] = 0

        # meta (present)

        meta = source.create_group("/present")
        meta.attrs["version"] = 0

        meta = other.create_group("/present")
        meta.attrs["version"] = 1


output = sorted(run("G5compare -c none --table PLAIN_COLUMNS a.hdf5 b.hdf5 -r /d/equal /e/equal"))
output = [i.strip() for i in output]

out = prettytable.PrettyTable()
out.set_style(prettytable.PLAIN_COLUMNS)
out.field_names = ["a.hdf5", "", "b.hdf5"]
out.align = "l"
out.add_row(["/a/not_equal", "!=", "/a/not_equal"])
out.add_row(["/b/not_equal", "!=", "/b/not_equal"])
out.add_row(["/c/not_equal", "!=", "/c/not_equal"])
out.add_row(["/f/not_equal", "!=", "/f/not_equal"])
out.add_row(["/present", "!=", "/present"])
out.add_row(["/meta", "->", ""])
expected_output = [i.strip() for i in sorted(out.get_string().split("\n"))]

os.remove("a.hdf5")
os.remove("b.hdf5")

for i in range(len(output)):
    if output[i] != expected_output[i]:
        print("|" + output[i] + "|")
        print("|" + expected_output[i] + "|")
        raise OSError("Test failed")



"""Try reading datasets.
    In case of reading failure the path is printed (otherwise nothing is printed).

:usage:

    G5check [options] <source>

:arguments:

    <source>
        HDF5-file.

:options:

    -b, --basic
        Only try getting a list of datasets, skip trying to read them.

    -h, --help
        Show help.

    --version
        Show version.

(c - MIT) T.W.J. de Geus | tom@geus.me | www.geus.me | github.com/tdegeus/GooseHDF5
"""
import argparse
import os
import warnings

import h5py

from .. import getdatapaths
from .. import verify
from .. import version

warnings.filterwarnings("ignore")


def check_isfile(fname):
    if not os.path.isfile(fname):
        raise OSError(f'"{fname}" does not exist')


def read(filename, check):

    with h5py.File(filename, "r") as source:
        paths = getdatapaths(source)
        if check:
            verify(source, paths, error=True)


def main():

    try:

        class Parser(argparse.ArgumentParser):
            def print_help(self):
                print(__doc__)

        parser = Parser()
        parser.add_argument("-b", "--basic", required=False, action="store_true")
        parser.add_argument("-v", "--version", action="version", version=version)
        parser.add_argument("source")
        args = parser.parse_args()

        check_isfile(args.source)
        read(args.source, not args.basic)

    except Exception as e:

        print(e)
        return 1


if __name__ == "__main__":

    main()



from pathlib import Path

from setuptools import find_packages
from setuptools import setup

project_name = "GooseHDF5"

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name=project_name,
    license="MIT",
    author="Tom de Geus",
    author_email="tom@geus.me",
    description="Wrapper around h5py",
    long_description=long_description,
    keywords="HDF5, h5py",
    url=f"https://github.com/tdegeus/{project_name:s}",
    packages=find_packages(),
    use_scm_version={"write_to": f"{project_name}/_version.py"},
    setup_requires=["setuptools_scm"],
    install_requires=["click", "h5py", "prettytable", "pyyaml", "termcolor"],
    entry_points={
        "console_scripts": [
            f"G5check = {project_name}.cli.G5check:main",
            f"G5compare = {project_name}:_G5compare_cli",
            f"G5list = {project_name}:_G5list_cli",
            f"G5print = {project_name}:_G5print_cli",
            f"G5repack = {project_name}.cli.G5repack:main",
            f"G5repair = {project_name}.cli.G5repair:main",
        ]
    },
)



import os
import pathlib
import shutil
import unittest

import h5py
import numpy as np

import GooseHDF5 as g5


basedir = pathlib.Path(__file__).parent / "mytest"


class Test_itereator(unittest.TestCase):
    """ """

    @classmethod
    def setUpClass(self):
        """
        Create a temporary directory for the test files.
        """
        os.makedirs(basedir, exist_ok=True)

    @classmethod
    def tearDownClass(self):
        """
        Remove the temporary directory.
        """
        shutil.rmtree(basedir)

    def test_copy_plain(self):

        datasets = ["/a", "/b/foo", "/c/d/foo"]

        with h5py.File(basedir / "foo_1.h5", "w") as source:
            with h5py.File(basedir / "bar_1.h5", "w") as dest:

                for d in datasets:
                    source[d] = np.random.rand(10)

                g5.copy(source, dest, datasets)

                for path in datasets:
                    self.assertTrue(g5.equal(source, dest, path))

    def test_copy_softlinks(self):

        datasets = ["/a", "/b/foo", "/c/d/foo"]
        links = ["/mylink/a", "/mylink/b/foo", "/mylink/c/d/foo"]

        with h5py.File(basedir / "foo_1.h5", "w") as source:
            with h5py.File(basedir / "bar_1.h5", "w") as dest:

                for link, d in zip(links, datasets):
                    source[d] = np.random.rand(10)
                    source[link] = h5py.SoftLink(d)

                g5.copy(source, dest, datasets + links, expand_soft=False)

                for path in datasets + links:
                    self.assertTrue(g5.equal(source, dest, path))
                for path in datasets:
                    self.assertNotIsInstance(dest.get(path, getlink=True), h5py.SoftLink)
                for path in links:
                    self.assertIsInstance(dest.get(path, getlink=True), h5py.SoftLink)

    def test_copy_expand_softlinks(self):

        datasets = ["/a", "/b/foo", "/c/d/foo"]
        links = ["/mylink/a", "/mylink/b/foo", "/mylink/c/d/foo"]

        with h5py.File(basedir / "foo_1.h5", "w") as source:
            with h5py.File(basedir / "bar_1.h5", "w") as dest:

                for link, d in zip(links, datasets):
                    source[d] = np.random.rand(10)
                    source[link] = h5py.SoftLink(d)

                g5.copy(source, dest, datasets + links)

                for path in datasets + links:
                    self.assertTrue(g5.equal(source, dest, path))
                for path in datasets + links:
                    self.assertNotIsInstance(dest.get(path, getlink=True), h5py.SoftLink)

    def test_copy_skip(self):

        datasets = ["/a", "/b/foo", "/c/d/foo"]

        with h5py.File(basedir / "foo_1.h5", "w") as source:
            with h5py.File(basedir / "bar_1.h5", "w") as dest:

                for d in datasets:
                    source[d] = np.random.rand(10)

                g5.copy(source, dest, datasets + ["/nonexisting"], skip=True)

                for path in datasets:
                    self.assertTrue(g5.equal(source, dest, path))

    def test_copy_nonrecursive(self):

        datasets = ["/a", "/b/foo", "/b/bar", "/c/d/foo"]

        with h5py.File(basedir / "foo_1.h5", "w") as source:
            with h5py.File(basedir / "bar_1.h5", "w") as dest:

                for d in datasets:
                    source[d] = np.random.rand(10)

                g5.copy(source, dest, ["/a", "/b", "/c/d/foo"], recursive=False)

                for path in ["/a", "/c/d/foo"]:
                    self.assertTrue(g5.equal(source, dest, path))

                self.assertTrue(g5.exists(dest, "/b"))
                self.assertFalse(g5.exists(dest, "/b/foo"))
                self.assertFalse(g5.exists(dest, "/b/bar"))

    def test_copy_recursive(self):

        datasets = ["/a", "/b/foo", "/b/bar", "/c/d/foo"]

        with h5py.File(basedir / "foo_1.h5", "w") as source:
            with h5py.File(basedir / "bar_1.h5", "w") as dest:

                for d in datasets:
                    source[d] = np.random.rand(10)

                g5.copy(source, dest, ["/a", "/b", "/c/d/foo"])

                for path in datasets:
                    self.assertTrue(g5.equal(source, dest, path))

    def test_copy_attrs(self):

        datasets = ["/a", "/b/foo", "/c/d/foo"]

        with h5py.File(basedir / "foo_1.h5", "w") as source:
            with h5py.File(basedir / "bar_1.h5", "w") as dest:

                for d in datasets:
                    source[d] = np.random.rand(10)

                meta = source.create_group("/meta")
                meta.attrs["version"] = np.random.rand(10)

                datasets += ["/meta"]
                g5.copy(source, dest, datasets)

                for path in datasets:
                    self.assertTrue(g5.equal(source, dest, path))

    def test_copy_groupattrs(self):

        datasets = ["/a", "/b/foo", "/c/d/foo"]

        with h5py.File(basedir / "foo_1.h5", "w") as source:
            with h5py.File(basedir / "bar_1.h5", "w") as dest:

                for d in datasets:
                    source[d] = np.random.rand(10)

                source["/b"].attrs["version"] = np.random.rand(10)

                datasets += ["/b"]
                g5.copy(source, dest, datasets)

                for path in datasets:
                    self.assertTrue(g5.equal(source, dest, path))

    def test_copy_root(self):

        datasets = ["/a", "/b/foo", "/c/d/foo"]
        source_pre = "/my/source"
        dest_pre = "/your/dest"

        with h5py.File(basedir / "foo_1.h5", "w") as source:
            with h5py.File(basedir / "bar_1.h5", "w") as dest:

                for d in datasets:
                    source[g5.join(source_pre, d)] = np.random.rand(10)

                g5.copy(source, dest, datasets, source_root=source_pre, root=dest_pre)

                for path in datasets:
                    s = g5.join(source_pre, path)
                    d = g5.join(dest_pre, path)
                    self.assertTrue(g5.equal(source, dest, s, d))


if __name__ == "__main__":

    unittest.main()
